const async_hooks = require('async_hooks');

const pre_storage = new Map();
const storage = new Map();
const requests_map = new Map();

function store(async_id, req) {
    storage.set(async_id, req);

    let req_map = requests_map.get(req);
    req_map.set(async_id, 0);
}

/**
 * Tracks the creation of new resources.
 * @param async_id
 * @param type
 * @param trigger_id
 */
function init(async_id, type, trigger_id) {
    if (pre_storage.has(async_hooks.executionAsyncId())) {
        store(async_id, pre_storage.get(async_hooks.executionAsyncId()));
    }

    if (storage.has(trigger_id)) {
        store(async_id, storage.get(trigger_id));
    }

    if (type === 'TickObject') {
        pre_storage.delete(trigger_id);
    }
}

/**
 * Tracks the destroy of async resources.
 * @param async_id
 */
function destroy(async_id) {
    storage.delete(async_id);
}

function startRoute(req) {
    requests_map.set(req, new Map());

    pre_storage.set(async_hooks.executionAsyncId(), req);
}

function stopRoute(req) {
    const req_map = requests_map.get(req);

    let iterator = req_map.keys();
    let key = iterator.next();

    while (!key.done) {
        storage.delete(key.value); // 1 3 5 7 9
        key = iterator.next();
    }
    requests_map.delete(req);
}

/**
 * Middleware for enable useRequest.
 * @param req
 * @param res
 * @param next
 */
function middleware(req, res, next) {
    let async_hook = async_hooks.createHook({init, destroy});
    async_hook.enable();
    startRoute(req);

    function onClose() {
        async_hook.disable();
        stopRoute(req);
    }

    res.on('finish', onClose);
    res.on('close', onClose);

    next();
}

/**
 * Returns the current request being processed.
 * @returns {http.ClientRequest|null}
 */
function useRequest() {
    return  storage.get(async_hooks.executionAsyncId()) || null;
}

module.exports.middleware = middleware;
module.exports.useRequest = useRequest;

